# -*- coding: utf-8 -*-
"""Combined_classification.ipynb

Automatically generated by Colaboratory.

Original file is located at
    https://colab.research.google.com/drive/18oDzOkExok8oXNpf4EuhMMqFtw4-uQdN
"""

import pandas as pd
import numpy as np
import pickle

import itertools
import xgboost as xgb
from sklearn.svm import LinearSVC
from sklearn.naive_bayes import MultinomialNB
from sklearn.neural_network import MLPClassifier
from sklearn.linear_model import LogisticRegression
from sklearn.ensemble import RandomForestClassifier
from sklearn.model_selection import train_test_split
from sklearn.feature_selection import SelectFromModel
from sklearn.feature_extraction.text import CountVectorizer

from sklearn.feature_extraction.text import TfidfTransformer, TfidfVectorizer
from sklearn.metrics import confusion_matrix, f1_score, accuracy_score, precision_score, recall_score

import matplotlib
from sklearn.manifold import TSNE
from sklearn.preprocessing import MinMaxScaler, OneHotEncoder
from sklearn.utils.class_weight import compute_class_weight

from IPython.display import display
import matplotlib.pyplot as plt
import seaborn as sns

x_train_text = pd.read_csv('/content/drive/MyDrive/CS626/iemocap/text_train.csv')
x_test_text = pd.read_csv('/content/drive/MyDrive/CS626/iemocap/text_test.csv')

y_train_text = x_train_text['label']
y_test_text = x_test_text['label']

x_train_audio = pd.read_csv('/content/drive/MyDrive/CS626/iemocap/audio_train.csv')
x_test_audio = pd.read_csv('/content/drive/MyDrive/CS626/iemocap/audio_test.csv')

y_train_audio = x_train_audio['label']
y_test_audio = x_test_audio['label']

y_train = y_train_audio  # since y_train_audio == y_train_text
y_test = y_test_audio  # since y_train_audio == y_train_text

print(x_train_text.shape, y_train_text.shape, x_train_audio.shape, y_train_audio.shape)

from google.colab import drive
drive.mount('/content/drive')

emotion_dict = {'ang': 0,
                'hap': 1,
                'sad': 2,
                'fea': 3,
                'sur': 4,
                'neu': 5}

emo_keys = list(['ang', 'hap', 'sad', 'fea', 'sur', 'neu'])

id_to_emotion = {0: 'ang', 1: 'hap', 2: 'sad', 3: 'fea', 4: 'sur', 5: 'neu'}

def plot_confusion_matrix(cm, classes,
                          normalize=False,
                          title='Confusion matrix',
                          cmap=plt.cm.Blues):
    """
    This function prints and plots the confusion matrix.
    Normalization can be applied by setting `normalize=True`.
    """
    # plt.figure(figsize=(8,8))
    plt.imshow(cm, interpolation='nearest', cmap=cmap)
    plt.title(title)
    plt.colorbar()
    tick_marks = np.arange(len(classes))
    plt.xticks(tick_marks, classes, rotation=45)
    plt.yticks(tick_marks, classes)

    if normalize:
        cm = cm.astype('float') / cm.sum(axis=1)[:, np.newaxis]
        print("Normalized confusion matrix")
    else:
        print('Confusion matrix, without normalization')

    print(cm)

    thresh = cm.max() / 2.
    for i, j in itertools.product(range(cm.shape[0]), range(cm.shape[1])):
        plt.text(j, i, cm[i, j],
                 horizontalalignment="center",
                 color="white" if cm[i, j] > thresh else "black")

    plt.tight_layout()
    plt.ylabel('True label')
    plt.xlabel('Predicted label')
    
def one_hot_encoder(true_labels, num_records, num_classes):
    temp = np.array(true_labels[:num_records])
    true_labels = np.zeros((num_records, num_classes))
    true_labels[np.arange(num_records), temp] = 1
    return true_labels

def display_results(y_test, pred_probs, cm=True):
    pred = np.argmax(pred_probs, axis=-1)
    one_hot_true = one_hot_encoder(y_test, len(pred), len(emotion_dict))
    print('Test Set Accuracy =  {0:.3f}'.format(accuracy_score(y_test, pred)))
    print('Test Set F-score =  {0:.3f}'.format(f1_score(y_test, pred, average='macro')))
    print('Test Set Precision =  {0:.3f}'.format(precision_score(y_test, pred, average='macro')))
    print('Test Set Recall =  {0:.3f}'.format(recall_score(y_test, pred, average='macro')))
    if cm:
        plot_confusion_matrix(confusion_matrix(y_test, pred), classes=emo_keys)

cl_weight = dict(pd.Series(x_train_audio['label']).value_counts(normalize=True))

"""## Get Text Features"""

tfidf = TfidfVectorizer(sublinear_tf=True, min_df=5, norm='l2', encoding='latin-1', ngram_range=(1, 2), stop_words='english')

features_text = tfidf.fit_transform(x_train_text.append(x_test_text).transcription).toarray()

x_train_text = features_text[:x_train_text.shape[0]]
x_test_text = features_text[-x_test_text.shape[0]:]

print(features_text.shape, x_train_text.shape, x_test_text.shape)

!pip install sentence_transformers

from sentence_transformers import SentenceTransformer
bert = SentenceTransformer('bert-base-uncased')

print(x_train_text)
# features = x_train_text[:10].transcription.apply(bert.encode)
features_text = x_train_text.append(x_test_text).transcription.apply(bert.encode)

x_train_text = features_text[:x_train_text.shape[0]]
x_test_text = features_text[-x_test_text.shape[0]:]

print(features_text.shape, x_train_text.shape, x_test_text.shape)
print(x_train_text)
x_train_text = np.array(x_train_text.values.tolist())
x_test_text = np.array(x_test_text.values.tolist())
print(features_text.shape, x_train_text.shape, x_test_text.shape)

"""## Combine Text + Audio Features"""

combined_x_train = np.concatenate((np.array(x_train_audio[x_train_audio.columns[2:]]), x_train_text), axis=1)
combined_x_test = np.concatenate((np.array(x_test_audio[x_test_audio.columns[2:]]), x_test_text), axis=1)

print(combined_x_train.shape, combined_x_test.shape)

combined_features_dict = {}

combined_features_dict['x_train'] = combined_x_train
combined_features_dict['x_test'] = combined_x_test
combined_features_dict['y_train'] = np.array(y_train)
combined_features_dict['y_test'] = np.array(y_test)

with open('/content/drive/MyDrive/CS626/iemocap/combined_features_bert.pkl', 'wb') as f:
    pickle.dump(combined_features_dict, f)

rf_classifier = RandomForestClassifier(n_estimators=600, min_samples_split=25)
rf_classifier.fit(combined_x_train, y_train)

# Predict
pred_probs = rf_classifier.predict_proba(combined_x_test)

# Results
display_results(y_test, pred_probs)

with open('/content/drive/MyDrive/CS626/iemocap/combined_rf_classifier_bert.pkl', 'wb') as f:
    pickle.dump(pred_probs, f)

with open('/content/drive/MyDrive/CS626/iemocap/RF_bert.pkl', 'wb') as f:
    pickle.dump(rf_classifier, f)

xgb_classifier = xgb.XGBClassifier(max_depth=7, learning_rate=0.008, objective='multi:softprob', 
                                   n_estimators=600, sub_sample=0.8, num_class=len(emotion_dict),
                                   booster='gbtree', n_jobs=4)
xgb_classifier.fit(combined_x_train, y_train)

# Predict
pred_probs = xgb_classifier.predict_proba(combined_x_test)

# Results
display_results(y_test, pred_probs)

with open('/content/drive/MyDrive/CS626/iemocap/combined_xgb_classifier_bert.pkl', 'wb') as f:
    pickle.dump(pred_probs, f)

with open('/content/drive/MyDrive/CS626/iemocap/XGB_bert.pkl', 'wb') as f:
    pickle.dump(xgb_classifier, f)

svc_classifier = LinearSVC()

svc_classifier.fit(combined_x_train, y_train)

# Predict
pred = svc_classifier.predict(combined_x_test)

# Results
one_hot_true = one_hot_encoder(y_test, len(pred), len(emotion_dict))
print('Test Set Accuracy =  {0:.3f}'.format(accuracy_score(y_test, pred)))
print('Test Set F-score =  {0:.3f}'.format(f1_score(y_test, pred, average='macro')))
print('Test Set Precision =  {0:.3f}'.format(precision_score(y_test, pred, average='macro')))
print('Test Set Recall =  {0:.3f}'.format(recall_score(y_test, pred, average='macro')))
plot_confusion_matrix(confusion_matrix(y_test, pred), classes=emo_keys)
(y_test, pred_probs)

with open('/content/drive/MyDrive/CS626/iemocap/combined_svc_classifier_model_bert.pkl', 'wb') as f:
    pickle.dump(pred, f)
    
with open('/content/drive/MyDrive/CS626/iemocap/SVC_bert.pkl', 'wb') as f:
    pickle.dump(svc_classifier, f)

mnb_classifier = MultinomialNB()

mnb_classifier.fit(combined_x_train, y_train)

# Predict
pred_probs = mnb_classifier.predict_proba(combined_x_test)

# Results
display_results(y_test, pred_probs)

with open('/content/drive/MyDrive/CS626/iemocap/combined_mnb_classifier_bert.pkl', 'wb') as f:
    pickle.dump(pred_probs, f)
    
with open('/content/drive/MyDrive/CS626/iemocap/MNB_bert.pkl', 'wb') as f:
    pickle.dump(mnb_classifier, f)

mlp_classifier = MLPClassifier(hidden_layer_sizes=(1000, ), activation='relu', solver='adam', alpha=0.0001,
                               batch_size='auto', learning_rate='adaptive', learning_rate_init=0.01,
                               power_t=0.5, max_iter=1000, shuffle=True, random_state=None, tol=0.0001,
                               verbose=False, warm_start=True, momentum=0.8, nesterovs_momentum=True,
                               early_stopping=False, validation_fraction=0.1, beta_1=0.9, beta_2=0.999,
                               epsilon=1e-08)

mlp_classifier.fit(combined_x_train, y_train)

# Predict
pred_probs = mlp_classifier.predict_proba(combined_x_test)

# Results
display_results(y_test, pred_probs)

with open('/content/drive/MyDrive/CS626/iemocap/combined_mlp_classifier_bert.pkl', 'wb') as f:
    pickle.dump(pred_probs, f)
    
with open('/content/drive/MyDrive/CS626/iemocap/MLP_bert.pkl', 'wb') as f:
    pickle.dump(mlp_classifier, f)

lr_classifier = LogisticRegression(solver='lbfgs', multi_class='multinomial', max_iter=1000)

lr_classifier.fit(combined_x_train, y_train)

# Predict
pred_probs = lr_classifier.predict_proba(combined_x_test)

# Results
display_results(y_test, pred_probs)

with open('/content/drive/MyDrive/CS626/iemocap/combined_lr_classifier_bert.pkl', 'wb') as f:
    pickle.dump(pred_probs, f)
    
with open('/content/drive/MyDrive/CS626/iemocap/LR_bert.pkl', 'wb') as f:
    pickle.dump(lr_classifier, f)

ax = xgb.plot_importance(xgb_classifier, max_num_features=10, height=0.5, show_values=False)
fig = ax.figure
fig.set_size_inches(8, 8)
contribution_scores = xgb_classifier.feature_importances_
print(contribution_scores)

# Load predicted probabilities
with open('/content/drive/MyDrive/CS626/iemocap/combined_rf_classifier_bert.pkl', 'rb') as f:
    rf_pred_probs = pickle.load(f)

with open('/content/drive/MyDrive/CS626/iemocap/combined_xgb_classifier_bert.pkl', 'rb') as f:
    xgb_pred_probs = pickle.load(f)
    
# with open('/content/drive/MyDrive/CS626/iemocap/combined_svc_classifier_model_bert.pkl', 'rb') as f:
#     svc_preds = pickle.load(f)
#     svc_preds = svc_preds.reshape(-1, 6)
# with open('/content/drive/MyDrive/CS626/iemocap/combined_mnb_classifier_bert.pkl', 'rb') as f:
#     mnb_pred_probs = pickle.load(f)
    
with open('/content/drive/MyDrive/CS626/iemocap/combined_mlp_classifier_bert.pkl', 'rb') as f:
    mlp_pred_probs = pickle.load(f)
    
with open('/content/drive/MyDrive/CS626/iemocap/combined_lr_classifier.pkl', 'rb') as f:
    lr_pred_probs = pickle.load(f)

# with open('/content/drive/MyDrive/CS626/iemocap/combined_lstm_classifier.pkl', 'rb') as f:
#     lstm_pred_probs = pickle.load(f)

# Average of the predicted probabilites
ensemble_pred_probs = (xgb_pred_probs +
                       mlp_pred_probs +
                       rf_pred_probs + 
                      #  mnb_pred_probs +
                       lr_pred_probs)
# Show metrics
display_results(y_test, ensemble_pred_probs)

from transformers import AutoModelForSequenceClassification

model = AutoModelForSequenceClassification.from_pretrained("bert-base-cased", num_labels=2)

from transformers import TrainingArguments

training_args = TrainingArguments("test_trainer")

from transformers import Trainer

trainer = Trainer(
    model=model, args=training_args, train_dataset=combined_x_train, eval_dataset=combined_x_test
)

new_xtrain = np.concatenate(combined_x_train, y_train)

trainer.train()

import numpy as np
from datasets import load_metric

metric = load_metric("accuracy")

def compute_metrics(eval_pred):
    logits, labels = eval_pred
    predictions = np.argmax(logits, axis=-1)
    return metric.compute(predictions=predictions, references=labels)